@@ -12,10 +12,11 @@ import argparse
 from pathlib import Path
 import sys
 
+# Do PYTHONPATH insert() instead of append() to prefer any local repo checkout over any pip install
 PATH_TO_TSFPGA = Path(__file__).parent.parent.resolve()
-sys.path.append(str(PATH_TO_TSFPGA))
+sys.path.insert(0, str(PATH_TO_TSFPGA))
 PATH_TO_VUNIT = PATH_TO_TSFPGA.parent / "vunit"
-sys.path.append(str(PATH_TO_VUNIT))
+sys.path.insert(0, str(PATH_TO_VUNIT))
 
 from examples.tsfpga_example_env import get_tsfpga_modules, TSFPGA_EXAMPLES_TEMP_DIR
 
