@@ -11,10 +11,11 @@ from shutil import which
 import subprocess
 import sys
 
+# Do PYTHONPATH insert() instead of append() to prefer any local repo checkout over any pip install
 PATH_TO_TSFPGA = Path(__file__).parent.parent.resolve()
-sys.path.append(str(PATH_TO_TSFPGA))
+sys.path.insert(0, str(PATH_TO_TSFPGA))
 PATH_TO_VUNIT = PATH_TO_TSFPGA.parent / "vunit"
-sys.path.append(str(PATH_TO_VUNIT))
+sys.path.insert(0, str(PATH_TO_VUNIT))
 
 from vunit import VUnitCLI, VUnit
 from vunit.vivado.vivado import create_compile_order_file, add_from_compile_order_file
@@ -60,7 +61,9 @@ def main():
         ip_core_vivado_project_directory=ip_core_vivado_project_directory,
     )
 
-    if vunit_proj.simulator_supports_coverage():
+    # Utilize coverage only for GHDL by default, since even if a commercial simulator supports
+    # coverage we do not know if we have a license for it.
+    if vunit_proj.get_simulator_name() == "ghdl" and vunit_proj.simulator_supports_coverage():
         vunit_proj.set_compile_option("enable_coverage", True)
         vunit_proj.set_sim_option("enable_coverage", True)
 
