@@ -1,6 +1,10 @@
-# ------------------------------------------------------------------------------
+# --------------------------------------------------------------------------------------------------
 # Copyright (c) Lukas Vik. All rights reserved.
-# ------------------------------------------------------------------------------
+#
+# This file is part of the tsfpga project.
+# https://tsfpga.com
+# https://gitlab.com/tsfpga/tsfpga
+# --------------------------------------------------------------------------------------------------
 
 
 class BuildStepTclHook:
@@ -11,7 +15,7 @@ class BuildStepTclHook:
 
     def __init__(self, tcl_file, hook_step):
         """
-        Args:
+        Arguments:
             tcl_file (`pathlib.Path`): Path to a TCL file.
             hook_step (str): Name of a build step, e.g. "STEPS.ROUTE_DESIGN.TCL.PRE".
         """
