@@ -15,11 +15,6 @@ from .register_code_generator import RegisterCodeGenerator
 class RegisterVhdlGenerator(RegisterCodeGenerator):
     """
     Generate a VHDL package with register information.
-
-    There is no unit test of this class that checks the generated code. It is instead functionally
-    tested in the file tb_artyz.vhd. That testbench compiles the VHDL package and performs
-    some run-time assertions on the generated values.
-    That test is considered more meaningful and exhaustive than a unit test would be.
     """
 
     def __init__(self, module_name, generated_info):
@@ -42,7 +37,7 @@ class RegisterVhdlGenerator(RegisterCodeGenerator):
     def _register_function_signature(self, register, register_array):
         return (
             "function "
-            f"{self._register_name(register, register_array)}(array_index : natural) return integer"
+            f"{self._register_name(register, register_array)}(array_index : natural) return natural"
         )
 
     def _register_indexes(self, register_objects):
@@ -50,10 +45,12 @@ class RegisterVhdlGenerator(RegisterCodeGenerator):
         for register, register_array in self._iterate_registers(register_objects):
             if register_array is None:
                 vhdl += (
-                    f"  constant {self._register_name(register)} : integer := {register.index};\n"
+                    f"  constant {self._register_name(register)} : natural := {register.index};\n"
                 )
             else:
                 vhdl += f"  {self._register_function_signature(register, register_array)};\n"
+        if vhdl:
+            vhdl += "\n"
 
         return vhdl
 
@@ -65,48 +62,87 @@ class RegisterVhdlGenerator(RegisterCodeGenerator):
         for register_object in register_objects:
             if isinstance(register_object, RegisterArray):
                 constant = self._array_length_constant_name(register_object)
-                vhdl += f"  constant {constant} : integer := {register_object.length};\n"
+                vhdl += f"  constant {constant} : natural := {register_object.length};\n"
         if vhdl:
             vhdl += "\n"
 
         return vhdl
 
     def _register_map(self, register_objects):
+        if not register_objects:
+            # It is possible that we have constants but no registers
+            return ""
+
         map_name = f"{self.module_name}_reg_map"
+        range_name = f"{self.module_name}_reg_range"
+
+        last_index = register_objects[-1].index
+        vhdl = f"""\
+  -- Declare register map constants here, but define them in body.
+  -- This is done so that functions have been elaborated when they are called.
+  subtype {range_name} is natural range 0 to {last_index};
+  constant {map_name} : reg_definition_vec_t({range_name});
+
+  subtype {self.module_name}_regs_t is reg_vec_t({range_name});
+  constant {self.module_name}_regs_init : {self.module_name}_regs_t;
+
+  subtype {self.module_name}_reg_was_accessed_t is std_logic_vector({range_name});
+
+"""
+
+        return vhdl
+
+    def _register_map_body(self, register_objects):
+        if not register_objects:
+            # It is possible that we have constants but no registers
+            return ""
+
+        map_name = f"{self.module_name}_reg_map"
+        range_name = f"{self.module_name}_reg_range"
 
         register_definitions = []
         default_values = []
+        vhdl_array_index = 0
         for register_object in register_objects:
             if isinstance(register_object, Register):
                 idx = self._register_name(register_object)
-                register_definitions.append(f"  (idx => {idx}, reg_type => {register_object.mode})")
-                default_values.append(
-                    f"  std_logic_vector(to_signed({register_object.default_value}, 32))"
+                opening = f"{vhdl_array_index} => "
+
+                register_definitions.append(
+                    f"{opening}(idx => {idx}, reg_type => {register_object.mode})"
                 )
+                default_values.append(
+                    f"{opening}std_logic_vector(to_signed({register_object.default_value}, 32))"
+                )
+
+                vhdl_array_index = vhdl_array_index + 1
             else:
                 for array_index in range(register_object.length):
                     for register in register_object.registers:
                         idx = f"{self._register_name(register, register_object)}({array_index})"
+                        opening = f"{vhdl_array_index} => "
+
                         register_definitions.append(
-                            f"  (idx => {idx}, reg_type => {register.mode})"
+                            f"{opening}(idx => {idx}, reg_type => {register.mode})"
                         )
                         default_values.append(
-                            f"  std_logic_vector(to_signed({register.default_value}, 32))"
+                            f"{opening}std_logic_vector(to_signed({register.default_value}, 32))"
                         )
 
-        last_index = register_objects[-1].index
-        vhdl = f"  constant {map_name} : reg_definition_vec_t(0 to {last_index}) := (\n  "
-        vhdl += ",\n  ".join(register_definitions)
-        vhdl += "\n  );\n"
-        vhdl += "\n"
-        vhdl += f"  subtype {self.module_name}_regs_t is reg_vec_t({map_name}'range);\n"
-        vhdl += (
-            "  subtype "
-            f"{self.module_name}_reg_was_accessed_t is std_logic_vector({map_name}'range);\n"
-        )
-        vhdl += f"  constant {self.module_name}_regs_init : {self.module_name}_regs_t := (\n  "
-        vhdl += ",\n  ".join(default_values)
-        vhdl += "\n  );\n"
+                        vhdl_array_index = vhdl_array_index + 1
+
+        array_element_separator = ",\n    "
+        vhdl = f"""\
+  constant {map_name} : reg_definition_vec_t({range_name}) := (
+    {array_element_separator.join(register_definitions)}
+  );
+
+  constant {self.module_name}_regs_init : {self.module_name}_regs_t := (
+    {array_element_separator.join(default_values)}
+  );
+
+"""
+
         return vhdl
 
     def _register_fields(self, register_objects):
@@ -116,12 +152,12 @@ class RegisterVhdlGenerator(RegisterCodeGenerator):
                 name = f"{self._register_name(register, register_array)}_{field.name}"
 
                 if field.width == 1:
-                    vhdl += f"  constant {name} : integer := {field.base_index};\n"
+                    vhdl += f"  constant {name} : natural := {field.base_index};\n"
                 else:
-                    vhdl += (
-                        f"  subtype {name} is integer range "
-                        f"{field.width + field.base_index - 1} downto {field.base_index};\n"
-                    )
+                    vhdl += f"""\
+  subtype {name} is natural range {field.width + field.base_index - 1} downto {field.base_index};
+  constant {name}_width : positive := {field.width};
+"""
 
             if register.fields:
                 vhdl += "\n"
@@ -135,18 +171,16 @@ class RegisterVhdlGenerator(RegisterCodeGenerator):
                 num_registers = len(register_object.registers)
                 array_length = self._array_length_constant_name(register_object)
                 for register in register_object.registers:
-                    vhdl += f"  {self._register_function_signature(register, register_object)} is\n"
-                    vhdl += "  begin\n"
-                    vhdl += f"    assert array_index < {array_length} \n"
-                    vhdl += (
-                        '      report "Array index out of bounds: " & natural\'image(array_index)\n'
-                    )
-                    vhdl += "      severity failure;\n"
-                    vhdl += (
-                        f"    return {register_object.base_index} + "
-                        f"array_index * {num_registers} + {register.index};\n"
-                    )
-                    vhdl += "  end function;\n\n"
+                    vhdl += f"""\
+  {self._register_function_signature(register, register_object)} is
+  begin
+    assert array_index < {array_length}
+      report "Array index out of bounds: " & natural'image(array_index)
+      severity failure;
+    return {register_object.base_index} + array_index * {num_registers} + {register.index};
+  end function;
+
+"""
 
         return vhdl
 
@@ -157,6 +191,9 @@ class RegisterVhdlGenerator(RegisterCodeGenerator):
                 "  constant "
                 f"{self.module_name}_constant_{constant.name} : integer := {constant.value};\n"
             )
+        if vhdl:
+            vhdl += "\n"
+
         return vhdl
 
     def get_package(self, register_objects, constants):
@@ -174,16 +211,17 @@ use reg_file.reg_file_pkg.all;
 
 package {pkg_name} is
 
-{self._register_indexes(register_objects)}
+{self._register_indexes(register_objects)}\
 {self._array_constants(register_objects)}\
-{self._register_map(register_objects)}
+{self._register_map(register_objects)}\
 {self._register_fields(register_objects)}\
-{self._constants(constants)}
+{self._constants(constants)}\
 end package;
 
 package body {pkg_name} is
 
 {self._array_index_functions(register_objects)}\
+{self._register_map_body(register_objects)}\
 end package body;
 """
 
