@@ -7,7 +7,8 @@
 # --------------------------------------------------------------------------------------------------
 
 from pathlib import Path
-from unittest import mock, TestCase
+from unittest import TestCase
+from unittest.mock import ANY, patch
 
 import pytest
 
@@ -58,6 +59,20 @@ def test_file_list_filtering(tmp_path):
     assert set(files) == set(synth_files)
 
 
+def test_get_synthesis_files_calls_get_simulation_files_with_correct_arguments():
+    module = BaseModule(path=Path(), library_name="")
+    with patch("tsfpga.module.BaseModule.get_synthesis_files") as get_synthesis_files:
+        module.get_simulation_files(files_include=True, files_avoid=False, apa=123)
+        get_synthesis_files.assert_called_once_with(files_include=True, files_avoid=False, apa=123)
+
+
+def test_get_formal_files_calls_get_simulation_files_with_correct_arguments():
+    module = BaseModule(path=Path(), library_name="")
+    with patch("tsfpga.module.BaseModule.get_synthesis_files") as get_synthesis_files:
+        module.get_formal_files(files_include=True, files_avoid=False, apa=123)
+        get_synthesis_files.assert_called_once_with(files_include=True, files_avoid=False, apa=123)
+
+
 def test_scoped_constraints(tmp_path):
     module_path = tmp_path / "apa"
     create_file(module_path / "src" / "hest.vhd")
@@ -93,6 +108,32 @@ def test_test_case_name():
     )
 
 
+def test_getting_registers_calls_registers_hook(tmp_path):
+    with patch("tsfpga.module.from_toml", autospec=True) as from_toml, patch(
+        "tsfpga.module.BaseModule.registers_hook", autospec=True
+    ) as registers_hook:
+        create_file(tmp_path / "a" / "regs_a.toml")
+        module = BaseModule(path=tmp_path / "a", library_name="a")
+        registers = module.registers
+
+        # TOML file exists so register creation from TOML should run
+        from_toml.assert_called_once()
+        registers_hook.assert_called_once()
+        assert registers is not None
+
+    with patch("tsfpga.module.from_toml", autospec=True) as from_toml, patch(
+        "tsfpga.module.BaseModule.registers_hook", autospec=True
+    ) as registers_hook:
+        module = BaseModule(path=tmp_path / "b", library_name="b")
+        registers = module.registers
+
+        # TOML file does not exist, so register creation from TOML should not run
+        from_toml.assert_not_called()
+        # Register hook shall still run however
+        registers_hook.assert_called_once()
+        assert registers is None
+
+
 @pytest.mark.usefixtures("fixture_tmp_path")
 class TestGetModules(TestCase):
 
@@ -157,32 +198,32 @@ class Module(BaseModule):
             else:
                 assert False
 
-    @mock.patch("tsfpga.module.from_toml", autospec=True)
-    def test_register_object_creation_synthesis(self, from_toml):
+    @patch("tsfpga.module.from_toml", autospec=True)
+    def test_register_object_creation_called_when_getting_synthesis_files(self, from_toml):
         toml_file = create_file(self.tmp_path / "a" / "regs_a.toml")
 
         module = get_modules(self.modules_folders).get("a")
         module.get_synthesis_files()
         module.get_synthesis_files()
 
-        from_toml.assert_called_once_with("a", toml_file, mock.ANY)
+        from_toml.assert_called_once_with("a", toml_file, ANY)
 
-    @mock.patch("tsfpga.module.from_toml", autospec=True)
-    def test_register_object_creation_simulation(self, from_toml):
+    @patch("tsfpga.module.from_toml", autospec=True)
+    def test_register_object_creation_called_when_getting_simulation_files(self, from_toml):
         toml_file = create_file(self.tmp_path / "a" / "regs_a.toml")
 
         module = get_modules(self.modules_folders).get("a")
         module.get_simulation_files()
         module.get_simulation_files()
 
-        from_toml.assert_called_once_with("a", toml_file, mock.ANY)
+        from_toml.assert_called_once_with("a", toml_file, ANY)
 
-    @mock.patch("tsfpga.module.from_toml", autospec=True)
-    def test_register_object_creation_mixed(self, from_toml):
+    @patch("tsfpga.module.from_toml", autospec=True)
+    def test_register_object_creation_called_once_when_getting_mixed_files(self, from_toml):
         toml_file = create_file(self.tmp_path / "a" / "regs_a.toml")
 
         module = get_modules(self.modules_folders).get("a")
         module.get_synthesis_files()
         module.get_simulation_files()
 
-        from_toml.assert_called_once_with("a", toml_file, mock.ANY)
+        from_toml.assert_called_once_with("a", toml_file, ANY)
